@@ -243,7 +243,7 @@ const GIT_COMMIT: &str = git_version::git_version!(
         "--abbrev=40",
         "--exclude=*"
     ],
-    fallback = "unknown"
+    fallback = "release"
 );
 
 pub const GIT_VERSION: &str =
@@ -914,9 +914,11 @@ pub mod scouting {
 /// # }
 /// ```
 pub mod liveliness {
-    pub use crate::api::liveliness::{
-        Liveliness, LivelinessGetBuilder, LivelinessSubscriberBuilder, LivelinessToken,
-        LivelinessTokenBuilder, LivelinessTokenUndeclaration,
+    pub use crate::api::{
+        builders::liveliness::{
+            LivelinessGetBuilder, LivelinessSubscriberBuilder, LivelinessTokenBuilder,
+        },
+        liveliness::{Liveliness, LivelinessToken, LivelinessTokenUndeclaration},
     };
 }
 
@@ -1099,7 +1101,16 @@ pub mod shm {
             },
         },
     };
+
+    pub use crate::net::runtime::ShmProviderState;
 }
 
+/// Functionality for interrupting queries.
+#[zenoh_macros::unstable]
+pub mod cancellation {
+    pub use crate::api::cancellation::CancellationToken;
+    #[cfg(feature = "internal")]
+    pub use crate::api::cancellation::SyncGroupNotifier;
+}
 #[cfg(test)]
 mod tests;
