@@ -20,8 +20,6 @@ use std::{
 use async_trait::async_trait;
 use tokio_util::sync::CancellationToken;
 use zenoh_buffers::ZBuf;
-#[cfg(feature = "stats")]
-use zenoh_protocol::zenoh::reply::ReplyBody;
 use zenoh_protocol::{
     core::{key_expr::keyexpr, Encoding, WireExpr},
     network::{
@@ -42,6 +40,7 @@ use super::{
 use crate::{
     key_expr::KeyExpr,
     net::routing::{
+        dispatcher::local_resources::{LocalResourceInfoTrait, LocalResources},
         hat::{HatTrait, SendDeclare},
         router::{get_or_set_route, QueryRouteBuilder},
     },
@@ -389,72 +388,6 @@ fn get_query_route(
     compute_route()
 }
 
-#[cfg(feature = "stats")]
-macro_rules! inc_req_stats {
-    (
-        $face:expr,
-        $txrx:ident,
-        $space:ident,
-        $body:expr
-    ) => {
-        paste::paste! {
-            if let Some(stats) = $face.stats.as_ref() {
-                use zenoh_buffers::buffer::Buffer;
-                match &$body {
-                    zenoh_protocol::zenoh::RequestBody::Query(q) => {
-                        stats.[<$txrx _z_query_msgs>].[<inc_ $space>](1);
-                        stats.[<$txrx _z_query_pl_bytes>].[<inc_ $space>](
-                            q.ext_body.as_ref().map(|b| b.payload.len()).unwrap_or(0),
-                        );
-                    }
-                }
-            }
-        }
-    };
-}
-
-#[cfg(feature = "stats")]
-macro_rules! inc_res_stats {
-    (
-        $face:expr,
-        $txrx:ident,
-        $space:ident,
-        $body:expr
-    ) => {
-        paste::paste! {
-            if let Some(stats) = $face.stats.as_ref() {
-                use zenoh_buffers::buffer::Buffer;
-                match &$body {
-                    ResponseBody::Reply(r) => {
-                        stats.[<$txrx _z_reply_msgs>].[<inc_ $space>](1);
-                        let mut n = 0;
-                        match &r.payload {
-                            ReplyBody::Put(p) => {
-                                if let Some(a) = p.ext_attachment.as_ref() {
-                                   n += a.buffer.len();
-                                }
-                                n += p.payload.len();
-                            }
-                            ReplyBody::Del(d) => {
-                                if let Some(a) = d.ext_attachment.as_ref() {
-                                   n += a.buffer.len();
-                                }
-                            }
-                        }
-                        stats.[<$txrx _z_reply_pl_bytes>].[<inc_ $space>](n);
-                    }
-                    ResponseBody::Err(e) => {
-                        stats.[<$txrx _z_reply_msgs>].[<inc_ $space>](1);
-                        stats.[<$txrx _z_reply_pl_bytes>].[<inc_ $space>](
-                            e.payload.len()
-                        );
-                    }
-                }
-            }
-        }
-    };
-}
-
 #[allow(clippy::too_many_arguments)]
 pub fn route_query(tables_ref: &Arc<TablesLock>, face: &Arc<FaceState>, msg: &mut Request) {
     let rtables = zread!(tables_ref.tables);
@@ -471,13 +404,9 @@ pub fn route_query(tables_ref: &Arc<TablesLock>, face: &Arc<FaceState>, msg: &mu
             let expr = RoutingExpr::new(&prefix, msg.wire_expr.suffix.as_ref());
 
             #[cfg(feature = "stats")]
-            let admin = expr.key_expr().is_some_and(|ke| ke.starts_with("@/"));
+            let payload_observer = super::stats::PayloadObserver::new(msg, Some(&expr), &rtables);
             #[cfg(feature = "stats")]
-            if !admin {
-                inc_req_stats!(face, rx, user, msg.payload)
-            } else {
-                inc_req_stats!(face, rx, admin, msg.payload)
-            }
+            payload_observer.observe_payload(zenoh_stats::Rx, face, msg);
 
             if tables_ref.hat_code.ingress_filter(&rtables, face, &expr) {
                 let route = get_query_route(
@@ -526,12 +455,6 @@ pub fn route_query(tables_ref: &Arc<TablesLock>, face: &Arc<FaceState>, msg: &mu
                         QueryCleanup::spawn_query_clean_up_task(
                             &outface, tables_ref, outqid, timeout,
                         );
-                        #[cfg(feature = "stats")]
-                        if !admin {
-                            inc_req_stats!(outface, tx, user, msg.payload)
-                        } else {
-                            inc_req_stats!(outface, tx, admin, msg.payload)
-                        }
 
                         tracing::trace!(
                             "{}:{} Propagate query to {}:{}",
@@ -540,7 +463,7 @@ pub fn route_query(tables_ref: &Arc<TablesLock>, face: &Arc<FaceState>, msg: &mu
                             outface,
                             outqid
                         );
-                        outface.primitives.send_request(&mut Request {
+                        let msg = &mut Request {
                             id: outqid,
                             wire_expr: key_expr,
                             ext_qos: msg.ext_qos,
@@ -550,7 +473,11 @@ pub fn route_query(tables_ref: &Arc<TablesLock>, face: &Arc<FaceState>, msg: &mu
                             ext_budget: msg.ext_budget,
                             ext_timeout: msg.ext_timeout,
                             payload: msg.payload.clone(),
-                        });
+                        };
+                        if outface.primitives.send_request(msg) {
+                            #[cfg(feature = "stats")]
+                            payload_observer.observe_payload(zenoh_stats::Tx, &outface, msg);
+                        }
                     }
                 }
             } else {
@@ -592,13 +519,15 @@ pub(crate) fn route_send_response(
 ) {
     let queries_lock = zread!(tables_ref.queries_lock);
     #[cfg(feature = "stats")]
-    let admin = msg.wire_expr.as_str().starts_with("@/");
+    let tables = zread!(tables_ref.tables);
     #[cfg(feature = "stats")]
-    if !admin {
-        inc_res_stats!(face, rx, user, msg.payload)
-    } else {
-        inc_res_stats!(face, rx, admin, msg.payload)
-    }
+    let expr = tables
+        .get_mapping(face, &msg.wire_expr.scope, msg.wire_expr.mapping)
+        .map(|prefix| RoutingExpr::new(prefix, msg.wire_expr.suffix.as_ref()));
+    #[cfg(feature = "stats")]
+    let payload_observer = super::stats::PayloadObserver::new(msg, expr.as_ref(), &tables);
+    #[cfg(feature = "stats")]
+    payload_observer.observe_payload(zenoh_stats::Rx, face, msg);
 
     match face.pending_queries.get(&msg.rid) {
         Some((query, _)) => {
@@ -613,15 +542,11 @@ pub(crate) fn route_send_response(
 
             drop(queries_lock);
 
-            #[cfg(feature = "stats")]
-            if !admin {
-                inc_res_stats!(query.src_face, tx, user, msg.payload)
-            } else {
-                inc_res_stats!(query.src_face, tx, admin, msg.payload)
-            }
-
             msg.rid = query.src_qid;
-            query.src_face.primitives.send_response(msg);
+            if query.src_face.primitives.send_response(msg) {
+                #[cfg(feature = "stats")]
+                payload_observer.observe_payload(zenoh_stats::Tx, &query.src_face, msg);
+            }
         }
         None => tracing::warn!("{}:{} Route reply: Query not found!", face, msg.rid),
     }
@@ -728,3 +653,39 @@ pub(crate) fn update_queryable_info(
         false
     }
 }
+
+impl LocalResourceInfoTrait<Arc<Resource>> for QueryableInfoType {
+    fn aggregate(
+        self_val: Option<Self>,
+        self_res: &Arc<Resource>,
+        other_val: &Self,
+        other_res: &Arc<Resource>,
+    ) -> Self {
+        // shortcut to avoid checking inclusion of ke, since we only care about completeness in aggregates and can ignore distance
+        if let Some(val) = self_val {
+            if val.complete == other_val.complete {
+                return val;
+            }
+        }
+
+        let other_complete = if other_val.complete {
+            if let (Some(self_ke), Some(other_ke)) = (self_res.keyexpr(), other_res.keyexpr()) {
+                other_ke.includes(self_ke)
+            } else {
+                false
+            }
+        } else {
+            false
+        };
+        let mut other_val = *other_val;
+        other_val.complete = other_complete;
+
+        if let Some(val) = self_val {
+            merge_qabl_infos(val, &other_val)
+        } else {
+            other_val
+        }
+    }
+}
+
+pub(crate) type LocalQueryables = LocalResources<QueryableId, Arc<Resource>, QueryableInfoType>;
